﻿
Type.registerNamespace("Demo");

Demo.Map = function(service, mapArgs) {
    /// <summary>
    ///   The VE Map.
    ///   Supports load on demand
    /// </summary>
    /// <param name="service">The webservice to call for the map data.</param>
    /// <param name="mapArgs">The map initalisation data.</param>

    this._service = service;
    this._mapArgs = mapArgs;
    
    this._map = null;
    this._layer = null;  
    
    this._OnMapChangeDelegate = null;
    
    this._init();
}

Demo.Map.prototype = {

    _init: function() {
        /// <summary>
        ///   Initialises the Map.
        /// </summary>       
        
        //setup map
        this._map = new VEMap(this._mapArgs.DivID);    
        this._map.LoadMap(this._mapArgs.Center,this._mapArgs.Zoomlevel,this._mapArgs.Style,this._mapArgs.Fixed,this._mapArgs.Mode); 
        this._map.SetScaleBarDistanceUnit(this._mapArgs.Scale);
        
        this._layer = new VEShapeLayer();         
        this._map.AddShapeLayer(this._layer);
        
        //setup the function to get new data whenever the map changes
        this._OnMapChangeDelegate = Function.createDelegate(this, this._GetPinData);
        this._map.AttachEvent("onchangeview", this._OnMapChangeDelegate);

        //get the data for the default view
        this._GetPinData();
    },         

    _GetPinData: function() {  
        /// <summary>
        ///   Get the latest map data from the webservice.
        /// </summary>

        var WKTBoundsEast = "";
        var WKTBoundsWest = "";
        var viewrect;
        
        if (this._map.GetMapStyle() == VEMapStyle.Birdseye) {    
            var be = this._map.GetBirdseyeScene();
            viewrect = be.GetBoundingRectangle();
        }else {
            viewrect = this._map.GetMapView();
        }
        if (this._map.GetMapMode() == VEMapMode.Mode3D) { 
            //we have four corners to use, to support view greater then half earth we would need to impliment logic as per 2D.           
            WKTBoundsEast = "POLYGON((" + viewrect.TopLeftLatLong.Latitude + " " + viewrect.TopLeftLatLong.Longitude + ", " + viewrect.BottomLeftLatLong.Latitude + " " + viewrect.BottomLeftLatLong.Longitude + ", " + viewrect.BottomRightLatLong.Latitude + " " + viewrect.BottomRightLatLong.Longitude + ", " + viewrect.TopRightLatLong.Latitude + " " + viewrect.TopRightLatLong.Longitude + ", "  + viewrect.TopLeftLatLong.Latitude + " " + viewrect.TopLeftLatLong.Longitude + "))";
        }else {
            //only have two points to use. Need to break into polygon no greater then half the Earth.
            if (viewrect.TopLeftLatLong.Longitude < 0) {
                if (viewrect.BottomRightLatLong.Longitude < 0) {
                    WKTBoundsWest = "POLYGON((" + viewrect.TopLeftLatLong.Latitude + " " + viewrect.TopLeftLatLong.Longitude + ", " + viewrect.BottomRightLatLong.Latitude + " " + viewrect.TopLeftLatLong.Longitude + ", " + viewrect.BottomRightLatLong.Latitude + " " + viewrect.BottomRightLatLong.Longitude + ", " + viewrect.TopLeftLatLong.Latitude + " " + viewrect.BottomRightLatLong.Longitude + ", "  + viewrect.TopLeftLatLong.Latitude + " " + viewrect.TopLeftLatLong.Longitude + "))";
                }else {
                    WKTBoundsWest = "POLYGON((" + viewrect.TopLeftLatLong.Latitude + " " + viewrect.TopLeftLatLong.Longitude + ", " + viewrect.BottomRightLatLong.Latitude + " " + viewrect.TopLeftLatLong.Longitude + ", " + viewrect.BottomRightLatLong.Latitude + " " + 0 + ", " + viewrect.TopLeftLatLong.Latitude + " " + 0 + ", "  + viewrect.TopLeftLatLong.Latitude + " " + viewrect.TopLeftLatLong.Longitude + "))";
                }
            }
            if (viewrect.BottomRightLatLong.Longitude > 0) {
                if (viewrect.TopLeftLatLong.Longitude > 0) {
                    WKTBoundsEast = "POLYGON((" + viewrect.TopLeftLatLong.Latitude + " " + viewrect.TopLeftLatLong.Longitude + ", " + viewrect.BottomRightLatLong.Latitude + " " + viewrect.TopLeftLatLong.Longitude + ", " + viewrect.BottomRightLatLong.Latitude + " " + viewrect.BottomRightLatLong.Longitude + ", " + viewrect.TopLeftLatLong.Latitude + " " + viewrect.BottomRightLatLong.Longitude + ", "  + viewrect.TopLeftLatLong.Latitude + " " + viewrect.TopLeftLatLong.Longitude + "))";
                }else {
                    WKTBoundsEast = "POLYGON((" + viewrect.TopLeftLatLong.Latitude + " " + 0 + ", " + viewrect.BottomRightLatLong.Latitude + " " + 0 + ", " + viewrect.BottomRightLatLong.Latitude + " " + viewrect.BottomRightLatLong.Longitude + ", " + viewrect.TopLeftLatLong.Latitude + " " + viewrect.BottomRightLatLong.Longitude + ", "  + viewrect.TopLeftLatLong.Latitude + " " + 0 + "))";
                }
            }        
        }
      
        //call webservice
        this._service.GetMapData(WKTBoundsEast, WKTBoundsWest, Function.createDelegate(this, this._OnMapDataSucceeded), Function.createDelegate(this, this._OnFailed));

    },

    _OnMapDataSucceeded: function(results) {
        /// <summary>
        ///   Receive data for map.
        /// </summary>  
        /// <param name="result">The webservice result object - JSON</param>  
        
        //delete old data
        this._layer.DeleteAllShapes();
                  
        //add new pins
        var newShapes = new Array();
        for(x = 0; x < results.length; x++) {
            var newShape = new VEShape(VEShapeType.Pushpin, new VELatLong(results[x].Latitude, results[x].Longitude));
            newShape.SetTitle(results[x].Title);
            newShape.SetDescription(results[x].Description);
            newShape.SetCustomIcon("pin.png");
            newShapes.push(newShape); 
        }
        
        this._layer.AddShape(newShapes);             
    },
    
    _OnFailed: function(error) {
        /// <summary>
        ///     This is the failed callback function for all webservices.
        /// </summary>  
        /// <param name="error">The error object from the webservice</param>          
        
        var stackTrace = error.get_stackTrace();
        var message = error.get_message();
        var statusCode = error.get_statusCode();
        var exceptionType = error.get_exceptionType();
        var timedout = error.get_timedOut();
       
        // Display the error.    
        var RsltElem = 
            "Stack Trace: " +  stackTrace + "<br/>" +
            "Service Error: " + message + "<br/>" +
            "Status Code: " + statusCode + "<br/>" +
            "Exception Type: " + exceptionType + "<br/>" +
            "Timedout: " + timedout;
            
            alert(RsltElem);
    },            

    Dispose: function() {
        /// <summary>
        ///   cleans up all objects. Detaches all events.
        /// </summary>
        if (this._map != null) {
            this._map.DetachEvent("onchangeview", this._OnMapChangeDelegate);              
            this._map.Dispose();
        }    
        this._service = null;
        this._mapArgs = null;
        
        this._map = null;
        this._layer = null;

        this._OnMapChangeDelegate = null;  
    }    
}

Demo.Map.registerClass('Demo.Map');

if (typeof(Sys) !== "undefined") Sys.Application.notifyScriptLoaded();

